/**
 * PollingEdit component.
 * 
 * Component provides polling creation/edit functionality.
 * 
 * @namespace Alfresco
 * @class Alfresco.PollingEdit
 */
(function()
{
   /**
    * YUI Library aliases
    */
   var Dom = YAHOO.util.Dom,
       Event = YAHOO.util.Event,
       Element = YAHOO.util.Element;

   /**
    * Alfresco Slingshot aliases
    */
   var $html = Alfresco.util.encodeHTML;
    
   /**
    * PollingEdit constructor.
    * 
    * @param {String} htmlId The HTML id of the parent element
    * @return {Alfresco.PollingEdit} The new PollingEdit instance
    * @constructor
    */
   Alfresco.PollingEdit = function(htmlId)
   {
      return Alfresco.PollingEdit.superclass.constructor.call(this, "Alfresco.PollingEdit", htmlId, ["json", "button", "menu"]);
   };
   
   YAHOO.extend(Alfresco.PollingEdit, Alfresco.component.Base,
   {
      /**
       * Object container for initialization options
       *
       * @property options
       * @type object
       */
      options:
      {
         /**
          * Current siteId.
          * 
          * @property siteId
          * @type string
          */
         siteId: "",
         
         /**
          * ContainerId representing root container
          *
          * @property containerId
          * @type string
          * @default "pollings"
          */
         containerId: "pollings",
         
         /**
          * True if the component should be in edit mode.
          */
         editMode: false,
         
         /**
          * Id of the polling to edit. Only relevant if editMode is true
          */
         pollingId: ""
      },

      /**
       * Stores the data of the currently edited polling
       */
      pollingData: null,
      
      /**
       * Fired by YUI when parent element is available for scripting.
       * Component initialisation, including instantiation of YUI widgets and event listener binding.
       *
       * @method onReady
       */
      onReady: function PollingEdit_onReady()
      {
         if (this.options.editMode)
         {
            // load the polling data prior to initializing the form
            this._loadPollingData();
         }
         
         this._initializePollingForm();

         // initialize form editor dialogs
         this._initializeFormEditor();
      },

      /**
       * Loads polling data
       */
      _loadPollingData: function PollingEdit__loadPollingData()
      {
		  // [TODO]
      },

      /**
       * Initializes the polling form with create/edit dependent data.
       */
      _initializePollingForm: function PollingEdit__initializePollingForm()
      {
         // construct the actionUrl, which is different for creating/updating a polling
         var actionUrl = "";
         if (this.options.editMode){
            actionUrl = YAHOO.lang.substitute(Alfresco.constants.PROXY_URI + "api/pollings/site/{site}/{container}/{pollingId}",
            {
               site: this.options.siteId,
               container : this.options.containerId,
               pollingId : this.options.pollingId
            });
         }
         else
         {
            actionUrl = YAHOO.lang.substitute(Alfresco.constants.PROXY_URI + "api/pollings/site/{site}/{container}",
            {
               site: this.options.siteId,
               container : this.options.containerId
            });
         }
         var form = Dom.get(this.id + '-form');
         form.setAttribute("action", actionUrl);

         // title
         var title = '';
         if (this.options.editMode)
         {
            title = this.pollingData.title;
         }
         Dom.get(this.id + '-title').setAttribute("value", title);
         
         // description
         var description = '';
         if (this.options.editMode)
         {
            description = $html(this.pollingData.description);
         }
         Dom.get(this.id + '-description').value = description;

         // polling item type
         var pollingItemType = '';
         if (this.options.editMode)
         {
            pollingItemType = this.pollingData.pollingItemType;
         }
         Dom.get(this.id + '-pollingItemType').value = pollingItemType;

         Dom.get(this.id + '-tag-input-field').tabIndex = 5;
         Dom.get(this.id + "-add-tag-button").tabIndex = 6;
         
         // register the behaviour with the form and display the form
         this._registerPollingForm();
      },
	  
      /**
       * Registers the form logic
       */
      _registerPollingForm: function PollingEdit__registerPollingForm()
      {
         // initialize the tag library
         this.modules.tagLibrary = new Alfresco.module.TagLibrary(this.id);
         this.modules.tagLibrary.setOptions(
         {
            siteId: this.options.siteId
         });
         
         // add the tags that are already set on the polling
         if (this.options.editMode && this.pollingData.tags.length > 0)
         {
            this.modules.tagLibrary.setTags(this.pollingData.tags);
         }
         
         // create the Button
         var okButtonLabel = '';
         if (this.options.editMode)
         {
            okButtonLabel = this.msg('button.update');
         }
         else
         {
            okButtonLabel = this.msg('button.save');
         }
         this.widgets.okButton = new YAHOO.widget.Button(this.id + "-ok",
         {
            type: "submit",
            label: okButtonLabel
         });

         // cancel button
         this.widgets.cancelButton = Alfresco.util.createYUIButton(this, "cancel", this.onFormCancelButtonClick);

         // create the form that does the validation/submit
         this.widgets.pollingForm = new Alfresco.forms.Form(this.id + "-form");
		 
         // title
         this.widgets.pollingForm.addValidation(this.id + "-title", Alfresco.forms.validation.mandatory, null, "keyup");
         this.widgets.pollingForm.addValidation(this.id + "-title", Alfresco.forms.validation.length,
         {
            max: 256,
            crop: true
         }, "keyup");
         // description
         this.widgets.pollingForm.addValidation(this.id + "-description", Alfresco.forms.validation.length,
         {
            max: 256,
            crop: true
         }, "keyup");

         this.widgets.pollingForm.setShowSubmitStateDynamically(true, false);
         this.widgets.pollingForm.setSubmitElements(this.widgets.okButton);
         this.widgets.pollingForm.setAJAXSubmit(true,
         {
            successCallback:
            {
               fn: this.onFormSubmitSuccess,
               scope: this
            },
            failureMessage: this.msg("message.savepolling.failure"),
            failureCallback:
            {
               fn: this.onFormSubmitFailure,
               scope: this
            }
         });
         if (this.options.editMode)
         {
             this.widgets.pollingForm.setAjaxSubmitMethod(Alfresco.util.Ajax.PUT);
         }
         this.widgets.pollingForm.setSubmitAsJSON(true);
         this.widgets.pollingForm.doBeforeFormSubmit =
         {
            fn: function(form, obj)
            {
                // disable ui elements
               this.widgets.okButton.set("disabled", true);
               this.widgets.cancelButton.set("disabled", true);
               
               // update the tags set in the form
               this.modules.tagLibrary.updateForm(this.id + "-form", "tags");
               
               // show a wait message
               this.widgets.feedbackMessage = Alfresco.util.PopupManager.displayMessage(
               {
                  text: Alfresco.util.message(this.msg("message.submitting")),
                  spanClass: "wait",
                  displayTime: 0
               });
            },
            scope: this
         };
         this.modules.tagLibrary.initialize(this.widgets.pollingForm);
         this.widgets.pollingForm.init();
         
         // finally display the form
         var containerElem = YAHOO.util.Dom.get(this.id + "-div");
         Dom.removeClass(containerElem, "hidden");
         Dom.get(this.id + "-title").focus();
      },

      questionData: null,
      
      questions: [],
      
      _initializeFormEditor: function PollingEdit__initializeFormEditor()
      {
         this.widgets.addQuestionButton = new YAHOO.widget.Button(this.id + "-add-question",
         {
            onclick: { fn: this.onAddQuestionClick, scope: this }
         });
         this.updateQuestions();

         var me = this;
         var handleSubmit = function() {
            me.questionData.title = Dom.get(me.id + '-question').value;
            me.questionData.type = Dom.get(me.id + '-type').value;
            me.questions[me.questions.length] = me.questionData;
            me.updateQuestions();
            this.submit();
         };
         var handleCancel = function() {
            this.cancel();
         };
         
         this.widgets.addQuestionDialog = new YAHOO.widget.Dialog(this.id + "-add-question-dialog-div", {
               width : "420px",
               fixedcenter : true,
               visible : false,
               modal : true,
               hideaftersubmit : true,
               constraintoviewport : true,
               buttons : [ { text : "Add", handler : handleSubmit, isDefault : true },
                           { text : "Cancel", handler : handleCancel } ]
            });

         this.widgets.addAnswerButton = new YAHOO.widget.Button(this.id + "-add-answer",
         {
            onclick: { fn: this.onAddAnswerClick, scope: this }
         });
         
         this.updateAnswers();
         this.widgets.addQuestionDialog.render();
         
         var containerElem = YAHOO.util.Dom.get(this.id + "-add-question-dialog-div");
         Dom.removeClass(containerElem, "hidden");
      },
      
      onAddQuestionClick: function PollingEdit_onAddQuestionClick(e)
      {
         this.questionData = { title: null, type: null, answers: [] };
         this.widgets.addQuestionDialog.show();
      },

      onAddAnswerClick: function PollingEdit_onAddAnswerClick(e)
      {
         this.questionData.answers.push(Dom.get(this.id + '-answer').value);
         Dom.get(this.id + '-answer').value = null;
         this.updateAnswers();
      },
      
      onRemoveAnswerClick: function PollingEdit_onRemoveAnswerClick(e, idx)
      {
         this.questionData.answers.splice(idx, 1);
         this.updateAnswers();
      },
      
      updateAnswers: function PollingEdit_updateAnswers()
      {
          var html;
          if (this.questionData != null && this.questionData.answers.length > 0) {
             html = "<table>";
             for (i in this.questionData.answers)
             {
                html += "<tr>";
                html += "<td class=\"details\">" + this.questionData.answers[i] + "</td>";
                html += "<td class=\"actions\"><a id=\"" + this.id + "-remove-answer-" + i + "\" class=\"remove-answer\" title=\"" + "Remove" + "\"></a></td>";
                html += "</tr>";
             }
             html += "</table>";
             Dom.get(this.id + '-answers').innerHTML = html; 
             for (i in this.questionData.answers)
             {
                Event.on(this.id + "-remove-answer-" + i, "click", this.onRemoveAnswerClick, i, this);
             }
          } else {
             html = "<div>No answers.</div>";
             Dom.get(this.id + '-answers').innerHTML = html;
          }
      },
      
      onRemoveQuestionClick: function PollingEdit_onRemoveQuestionClick(e, idx)
      {
         this.questions.splice(idx, 1);
         this.updateAnswers();
      },

      updateQuestions: function PollingEdit_updateQuestions()
      {
          var html;
          if (this.questions.length > 0) {
             html = "<table>";
             for (i in this.questions)
             {
                html += "<tr>";
                html += "<td class=\"details\">" + this.questions[i].title + "</td>";
                html += "<td class=\"actions\"><a id=\"" + this.id + "-remove-question-" + i + "\" class=\"remove-question\" title=\"" + "Remove" + "\"></a></td>";
                html += "</tr>";
             }
             html += "</table>";
             Dom.get(this.id + '-questions').innerHTML = html; 
             for (i in this.questions)
             {
                Event.on(this.id + "-remove-question-" + i, "click", this.onRemoveQuestionClick, i, this);
             }
          } else {
             html = "<div>No questions.</div>";
             Dom.get(this.id + '-questions').innerHTML = html;
          }
      },
      
      /**
       * Cancel button click handler
       */
      onFormCancelButtonClick: function PollingEdit_onFormCancelButtonClick(type, args)
      {
         // redirect to the page we came from
         history.go(-1);
      },
      
      /**
       * Form submit success handler
       */
      onFormSubmitSuccess: function PollingEdit_onFormSubmitSuccess(response)
      {
         // hide the wait message
         this.widgets.feedbackMessage.destroy();
         Alfresco.util.PopupManager.displayMessage({text: this.msg("message.savepolling.success")});
         history.go(-1);
      },

      /**
       * Reenables the inputs which got disabled as part of a comment submit
       */
      onFormSubmitFailure: function PollingEdit_onFormSubmitFailure(response)
      {
         response.config.failureMessage = YAHOO.lang.JSON.parse(response.serverResponse.responseText).message;

         // enable the buttons
         this.widgets.okButton.set("disabled", false);
         this.widgets.cancelButton.set("disabled", false);
         
         // hide the wait message
         this.widgets.feedbackMessage.destroy();
      }, 
      
      /**
       * PRIVATE FUNCTIONS
       */
          
   });
})();
